# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function


class TCellPolicy(object):
    def __init__(self):
        self.policy_id = None
